use piccolo::{Callback, CallbackReturn, Context, FromValue, Table};

use crate::{echo_to_term_frame, id_intern::intern_id, GlobalMemoCell, TermFrame};

use super::try_unwrap_frame;

pub(super) fn new_frameroute<'gc>(
    ctx: Context<'gc>,
    _global_memo: &GlobalMemoCell,
) -> Callback<'gc> {
    Callback::from_fn(&ctx, move |ctx, _ex, mut stack| {
        let frameroute: Table = Table::from_value(
            ctx,
            stack
                .pop_front()
                .ok_or_else(|| anyhow::Error::msg("classes.frameroute:new missing object!"))?,
        )?;
        let frame: TermFrame = try_unwrap_frame(
            ctx,
            &stack
                .pop_front()
                .ok_or_else(|| anyhow::Error::msg("classes.frameroute:new missing frame!"))?,
        )?;
        // We re-intern it to ensure we always point at the userdata.
        let frame = intern_id(ctx, frame);

        frameroute.set(ctx, ctx.intern_static(b"frame"), frame)?;

        Ok(piccolo::CallbackReturn::Return)
    })
}

pub(super) fn frameroute_route<'gc>(
    ctx: Context<'gc>,
    global_memo: &GlobalMemoCell,
) -> Callback<'gc> {
    let global_memo = global_memo.clone();
    Callback::from_fn(&ctx, move |ctx, _ex, mut stack| {
        let frameroute = Table::from_value(
            ctx,
            stack
                .pop_front()
                .ok_or_else(|| anyhow::Error::msg("frameroute:route called without self!"))?,
        )?;
        let line = piccolo::String::from_value(
            ctx,
            stack
                .pop_front()
                .ok_or_else(|| anyhow::Error::msg("frameroute:route called without line!"))?,
        )?;
        let frame: TermFrame =
            try_unwrap_frame(ctx, &frameroute.get(ctx, ctx.intern_static(b"frame"))?)?;

        // We ignore errors with the term frame to avoid breaking the entire client for one closed frame.
        echo_to_term_frame(&global_memo, &frame, &String::from_utf8_lossy(&line)).unwrap_or(());
        Ok(CallbackReturn::Return)
    })
}
